@@ -365,18 +365,23 @@ setLavaanOptions <- function(opt = formals(lavaan))
         opt$missing <- "listwise"
     } else if(opt$estimator == "pml") {
         opt$estimator <- "PML"
-        if(opt$se != "none") opt$se <- "none" # should be "robust.huber.white"
+        opt$information <- "observed"
+        if(opt$se == "default")
+            opt$se <- "robust.huber.white"
         if(opt$test != "none") opt$test <- "standard"
         opt$missing <- "listwise"
     } else {
         stop("unknown value for `estimator' argument: ", opt$estimator, "\n")
     }
 
-    # likelihood approach (wishart or normal)
+    # likelihood approach (wishart or normal) + sample.cov.rescale
     if(opt$estimator != "ML") {
         if(opt$likelihood != "default") {
             stop("likelihood argument is only relevant if estimator = ML")
         } 
+        if(opt$sample.cov.rescale != "default") {
+            stop("sample.cov.rescale argument is only relevant if estimator = ML")
+        }
     } else { # ml
         if(opt$likelihood == "default") {
            opt$likelihood <- "normal"
@@ -391,6 +396,17 @@ setLavaanOptions <- function(opt = formals(lavaan))
             stop("invalid value for `likelihood' argument: ", 
                  opt$likelihood, "\n")
         }
+
+        if(opt$sample.cov.rescale == "default") {
+            opt$sample.cov.rescale <- FALSE
+            if(opt$likelihood == "normal") {
+                opt$sample.cov.rescale <- TRUE
+            }
+        } else if(!is.logical(opt$sample.cov.rescale)) {
+            stop("sample.cov.rescale must be either \"default\", TRUE, or FALSE")
+        } else {
+            # nothing to do
+        }
     }
 
     # information
@@ -411,7 +427,8 @@ setLavaanOptions <- function(opt = formals(lavaan))
 
     # fixed.x
     if(is.logical(opt$fixed.x)) {
-        # nothing to do
+        if(opt$estimator == "DWLS" && opt$fixed.x == FALSE)
+            stop("lavaan ERROR: fixed.x=FALSE is not supported for estimator DWLS")
     } else if(opt$fixed.x == "default") {
         if(opt$estimator == "ML" && (opt$mimic == "Mplus" || 
                                      opt$mimic == "lavaan")) {
