@@ -1,7 +1,7 @@
 fitMeasures <- fitmeasures <- function(object, fit.measures="all") {
 
     # has the model converged?
-    if(!object@Fit@converged) {
+    if(object@Fit@npar > 0L && !object@Fit@converged) {
         stop("lavaan ERROR: fit measures not available if model did not converge")
     }
 
@@ -34,6 +34,17 @@ fitMeasures <- fitmeasures <- function(object, fit.measures="all") {
     G <- object@Data@ngroups  # number of groups
     X2 <- object@Fit@test[[1]]$stat
     df <- object@Fit@test[[1]]$df
+
+    # fit stat and df are NA (perhaps test="none"?), try again:
+    if(is.na(df)) {
+        df <- getDF(object@ParTable)
+        if(nrow(object@Model@con.jac) > 0L) {
+            df <- ( df + length(attr(object@Model@con.jac, "ceq.idx")) )
+        }
+    }
+    #if(is.na(X2) && is.finite(fx)) {
+    #                
+    #}
    
     if(test %in% c("satorra.bentler", "yuan.bentler", 
                    "mean.var.adjusted", "scaled.shifted")) {
@@ -48,73 +59,100 @@ fitMeasures <- fitmeasures <- function(object, fit.measures="all") {
         df.scaled <- object@Fit@test[[2]]$df
     }
 
+    # define 'sets' of fit measures:
+
+    # basic chi-square test
+    fit.chisq <- c("fmin", "chisq", "df", "pvalue")
+    if(scaled) {
+        fit.chisq <- c(fit.chisq, "chisq.scaled", "df.scaled", "pvalue.scaled", 
+                       "chisq.scaling.factor")
+    }
+
+    # basline model
+    fit.baseline <- c("baseline.chisq", "baseline.df", "baseline.pvalue")
+    if(scaled) {
+        fit.baseline <- c(fit.baseline, "baseline.chisq.scaled", 
+                          "baseline.df.scaled", "baseline.pvalue.scaled",
+                          "baseline.chisq.scaling.factor")
+    }
+
+    # cfi/tli
+    fit.cfi.tli <- c("cfi", "tli")
+    if(scaled) {
+        fit.cfi.tli <- c(fit.cfi.tli, "cfi.scaled", "tli.scaled")
+    }
+
+    # more incremental fit indices
+    fit.incremental <- c("cfi", "tli", "nnfi", "rfi", "nfi", "pnfi",
+                         "ifi", "rni")
+    if(scaled) { 
+        fit.incremental <- c(fit.incremental, "cfi.scaled", "tli.scaled", 
+                             "nnfi.scaled", "rfi.scaled", "nfi.scaled", 
+                             "ifi.scaled", "rni.scaled")
+    }
+    
+    # likelihood based measures
+    fit.logl <- c("logl", "unrestricted.logl", "npar", "aic", "bic",
+                  "ntotal", "bic2")
+    if(scaled && object@Options$test == "yuan.bentler") {
+        fit.logl <- c(fit.logl, "scaling.factor.h1", "scaling.factor.h0")
+    }
+
+    # rmsea
+    fit.rmsea <- c("rmsea", "rmsea.ci.lower", "rmsea.ci.upper", "rmsea.pvalue")
+    if(scaled) {
+        fit.rmsea <- c(fit.rmsea, "rmsea.scaled", "rmsea.ci.lower.scaled", 
+                       "rmsea.ci.upper.scaled", "rmsea.pvalue.scaled")
+    }
+
+    # srmr
+    if(categorical) {
+        fit.srmr <- character(0L)
+        fit.srmr2 <- character(0L)
+    } else {
+        fit.srmr <- c("srmr")
+        fit.srmr2 <- c("rmr", "rmr_nomean", "srmr", "srmr_nomean")
+    }
+
+    # various
+    fit.other <- c("cn_05","cn_01","gfi","agfi","pgfi","mfi")
+    if(!categorical && G == 1) {
+        fit.other <- c(fit.other, "ecvi")
+    }
+
+
+    # lower case
+    fit.measures <- tolower(fit.measures)
 
     # select 'default' fit measures
-    fit.measures <- tolower(fit.measures)
-    if("all" %in% fit.measures) {
-        if(scaled) {
-            chisq <- c("chisq", "df", "pvalue", "chisq.scaled", "df.scaled",
-                       "pvalue.scaled", "chisq.scaling.factor")
-        } else {
-            chisq <- c("chisq", "df", "pvalue")
-        }
-        if(scaled) {
-            baseline <- c("baseline.chisq", "baseline.df", "baseline.pvalue",
-                          "baseline.chisq.scaled", "baseline.df.scaled", 
-                          "baseline.pvalue.scaled", 
-                          "baseline.chisq.scaling.factor")
-        } else {
-            baseline <- c("baseline.chisq", "baseline.df", "baseline.pvalue")
-        }
-        if(scaled) {
-            cfi.tli <- c("cfi", "tli", "cfi.scaled", "tli.scaled")
-        } else {
-            cfi.tli <- c("cfi", "tli")
-        }
-        if(scaled && object@Options$test == "yuan.bentler") {
-            logl <- c("logl", "unrestricted.logl", "npar", "aic", "bic",
-                      "scaling.factor.h1", "scaling.factor.h0", "ntotal")
-        } else {
-            logl <- c("logl", "unrestricted.logl", "npar", "aic", "bic", "ntotal")
-        }
-        #if(object@Options$mimic == "Mplus") {
-            logl <- c(logl, "bic2")
-        #}
-        if(scaled) {
-            rmsea <- c("rmsea", "rmsea.scaled")
-            rmsea.ci <- c("rmsea", "rmsea.ci.lower", "rmsea.ci.upper",
-                          "rmsea.scaled", "rmsea.ci.lower.scaled", 
-                           "rmsea.ci.upper.scaled")
-            rmsea.full <- c("rmsea", "rmsea.ci.lower", "rmsea.ci.upper", 
-                            "rmsea.pvalue",
-                            "rmsea.scaled", "rmsea.ci.lower.scaled", 
-                            "rmsea.ci.upper.scaled",
-                            "rmsea.pvalue.scaled")
-        } else {
-            rmsea <- c("rmsea")
-            rmsea.ci <- c("rmsea", "rmsea.ci.lower", "rmsea.ci.upper")
-            rmsea.full <- c("rmsea", 
-                            "rmsea.ci.lower", "rmsea.ci.upper", "rmsea.pvalue")
-        }
-
-        if(categorical) {
-            srmr <- character(0)
-        } else {
-            srmr <- c("srmr", "srmr_nomean")
-        }
-  
-
-        if(estimator == "ML") {
-            fit.measures <- c(chisq, baseline, cfi.tli, logl, rmsea.full, srmr)
-        } else {
-            fit.measures <- c(chisq, baseline, cfi.tli, rmsea.full, srmr)
+    if(length(fit.measures) == 1L) {
+        if(fit.measures == "default") {
+            if(estimator == "ML") {
+                fit.measures <- c(fit.chisq, fit.baseline, 
+                                  fit.cfi.tli, fit.logl, 
+                                  fit.rmsea, fit.srmr)
+            } else {
+                fit.measures <- c(fit.chisq, fit.baseline, fit.cfi.tli, 
+                                  fit.rmsea, fit.srmr)
+            }
+        } else if(fit.measures == "all") {
+            if(estimator == "ML") {
+                fit.measures <- c(fit.chisq, fit.baseline, fit.incremental, 
+                                  fit.logl, fit.rmsea, fit.srmr2, fit.other)
+            } else {
+                fit.measures <- c(fit.chisq, fit.baseline, fit.incremental,
+                                  fit.rmsea, fit.srmr2, fit.other)
+            }
         }
     }
+    
+    # main container
     indices <- list()
 
     # Chi-square value estimated model (H0)
-    if(any(c("chisq", "chisq.scaled", 
+    if(any(c("fmin", "chisq", "chisq.scaled", 
              "chisq.scaling.factor") %in% fit.measures)) {
+    indices["fmin"] <- fx
 	indices["chisq"] <- X2
         if(scaled) {
             indices["chisq.scaled"] <- X2.scaled
@@ -136,9 +174,12 @@ fitMeasures <- fitmeasures <- function(object, fit.measures="all") {
 
 
     if(any(c("cfi", "cfi.scaled", "tli", "tli.scaled",
+             "nnfi", "nnfi.scaled", "pnfi", "pnfi.scaled",
+             "rfi", "rfi.scaled", "nfi", "nfi.scaled",
+             "ifi", "ifi.scaled", "rni", "rni.scaled",
              "baseline.chisq", "baseline.chisq.scaled",
              "baseline.pvalue", "baseline.pvalue.scaled") %in% fit.measures)) {
-
+        
         # call explicitly independence model
         # this is not strictly needed for ML, but it is for
         # GLS and WLS
@@ -182,62 +223,194 @@ fitMeasures <- fitmeasures <- function(object, fit.measures="all") {
             df.null.scaled <- fit.indep@Fit@test[[2]]$df
         } 
 
-        if("baseline.chisq" %in% fit.measures) {
-            indices["baseline.chisq"] <- X2.null
-            if(scaled) {
-                indices["baseline.chisq.scaled"] <- X2.null.scaled
-            } 
-        }
-        if("baseline.df" %in% fit.measures) {
-            indices["baseline.df"] <- df.null
-            if(scaled) {
-                indices["baseline.df.scaled"] <- df.null.scaled
-            }
-        }
-        if("baseline.pvalue" %in% fit.measures) {
-            indices["baseline.pvalue"] <- fit.indep@Fit@test[[1]]$pvalue
-            if(scaled) {
-                indices["baseline.pvalue.scaled"] <- 
-                    fit.indep@Fit@test[[2]]$pvalue
-            }
-        }
-        if("baseline.chisq.scaling.factor" %in% fit.measures) {
-            indices["baseline.chisq.scaling.factor"] <-
-                fit.indep@Fit@test[[2]]$scaling.factor
-        }
+        # check for NAs
+        if(is.na(X2) || is.na(df) || is.na(X2.null) || is.na(df.null)) {
+            indices[fit.incremental] <- as.numeric(NA)
+        } else {
 
-        # CFI 
-        if("cfi" %in% fit.measures) {
-            indices["cfi"] <- ( 1 - max(c(X2 - df,0)) / 
-                                    max( c(X2-df, X2.null-df.null, 0) ) 
-                              )
-        }
-        if("cfi.scaled" %in% fit.measures) {
-            indices["cfi.scaled"] <- 
-                ( 1 - max( c(X2.scaled - df.scaled,0)) / 
-                      max( c(X2.scaled - df.scaled, 
-                             X2.null.scaled - df.null.scaled, 0) ) 
-                )
-        }
+            if("baseline.chisq" %in% fit.measures) {
+                indices["baseline.chisq"] <- X2.null
+                if(scaled) {
+                    indices["baseline.chisq.scaled"] <- X2.null.scaled
+                } 
+            }
+            if("baseline.df" %in% fit.measures) {
+                indices["baseline.df"] <- df.null
+                if(scaled) {
+                    indices["baseline.df.scaled"] <- df.null.scaled
+                }
+            }
+            if("baseline.pvalue" %in% fit.measures) {
+                indices["baseline.pvalue"] <- fit.indep@Fit@test[[1]]$pvalue
+                if(scaled) {
+                    indices["baseline.pvalue.scaled"] <- 
+                        fit.indep@Fit@test[[2]]$pvalue
+                }
+            }
+            if("baseline.chisq.scaling.factor" %in% fit.measures) {
+                indices["baseline.chisq.scaling.factor"] <-
+                    fit.indep@Fit@test[[2]]$scaling.factor
+            }
 
-        # TLI 
-        if("tli" %in% fit.measures) {
-            if(df > 0) {
-                TLI <- (X2.null/df.null - X2/df)/(X2.null/df.null - 1)
-            } else {
-                TLI <- 1
+            # CFI - comparative fit index (Bentler, 1990) 
+            if("cfi" %in% fit.measures) {
+                t1 <- max( c(X2 - df, 0) )
+                t2 <- max( c(X2 - df, X2.null - df.null, 0) )
+                if(t1 == 0 && t2 == 0) {
+                    indices["cfi"] <- 1
+                } else {
+                    indices["cfi"] <- 1 - t1/t2
+                }
             }
-            indices["tli"] <- TLI
-        }
-        if("tli.scaled" %in% fit.measures) {
-            if(df > 0) {
-                TLI <- (X2.null.scaled/df.null.scaled - 
-                        X2.scaled/df.scaled) / 
-                       (X2.null.scaled/df.null.scaled - 1)
-            } else {
-                TLI <- 1
+            if("cfi.scaled" %in% fit.measures) {
+                t1 <- max( c(X2.scaled - df.scaled, 0) )
+                t2 <- max( c(X2.scaled - df.scaled,
+                             X2.null.scaled - df.null.scaled, 0) )
+                if(t1 == 0 && t2 == 0) {
+                    indices["cfi.scaled"] <- 1
+                } else {
+                    indices["cfi.scaled"] <- 1 - t1/t2
+                }
+            }
+
+            # TLI - Tucker-Lewis index (Tucker & Lewis, 1973)
+            # same as
+            # NNFI - nonnormed fit index (NNFI, Bentler & Bonett, 1980)
+            if("tli" %in% fit.measures || "nnfi" %in% fit.measures) {
+                if(df > 0) {
+                    t1 <- X2.null/df.null - X2/df
+                    t2 <- X2.null/df.null - 1 
+                    # note: TLI original formula was in terms of fx/df, not X2/df
+                    # then, t1 <- fx_0/df.null - fx/df
+                    #       t2 <- fx_0/df.null - 1/N (or N-1 for wishart)
+                    if(t1 < 0 && t2 < 0) {
+                        TLI <- 1
+                    } else {
+                        TLI <- t1/t2
+                    }
+                } else {
+                   TLI <- 1
+                }
+                indices["tli"] <- indices["nnfi"] <- TLI
+            }
+            if("tli.scaled" %in% fit.measures || "nnfi.scaled" %in% fit.measures) {
+                if(df > 0) {
+                    t1 <- X2.null.scaled/df.null.scaled - X2.scaled/df.scaled
+                    t2 <- X2.null.scaled/df.null.scaled - 1
+                    if(t1 < 0 && t2 < 0) {
+                        TLI <- 1
+                    } else {
+                        TLI <- t1/t2
+                    }
+                } else {
+                    TLI <- 1
+                }
+                indices["tli.scaled"] <- indices["nnfi.scaled"] <- TLI
+            }
+    
+    
+            # RFI - relative fit index (Bollen, 1986; Joreskog & Sorbom 1993)
+            if("rfi" %in% fit.measures) {
+                if(df > 0) {
+                    t1 <- X2.null/df.null - X2/df
+                    t2 <- X2.null/df.null
+                    if(t1 < 0 || t2 < 0) {
+                        RLI <- 1
+                    } else {
+                        RLI <- t1/t2
+                    }
+                } else {
+                   RLI <- 1
+                }
+                indices["rfi"] <- RLI
+            }
+            if("rfi.scaled" %in% fit.measures) {
+                if(df > 0) {
+                    t1 <- X2.null.scaled/df.null.scaled - X2.scaled/df.scaled
+                    t2 <- X2.null.scaled/df.null.scaled
+                    if(t1 < 0 || t2 < 0) {
+                        RLI <- 1
+                    }     else {
+                        RLI <- t1/t2
+                    }
+                } else {
+                   RLI <- 1
+                }
+                indices["rfi.scaled"] <- RLI
+            }
+
+            # NFI - normed fit index (Bentler & Bonett, 1980)
+            if("nfi" %in% fit.measures) {
+                t1 <- X2.null - X2
+                t2 <- X2.null
+                NFI <- t1/t2
+                indices["nfi"] <- NFI
+            }
+            if("nfi.scaled" %in% fit.measures) {
+                t1 <- X2.null.scaled - X2.scaled
+                t2 <- X2.null.scaled
+                NFI <- t1/t2
+                indices["nfi.scaled"] <- NFI
+            }
+
+            # PNFI - Parsimony normed fit index (James, Mulaik & Brett, 1982)
+            if("pnfi" %in% fit.measures) {
+                t1 <- X2.null - X2
+                t2 <- X2.null
+                PNFI <- (df/df.null) * t1/t2
+                indices["pnfi"] <- PNFI
+            }
+            if("pnfi.scaled" %in% fit.measures) {
+                t1 <- X2.null.scaled - X2.scaled
+                t2 <- X2.null.scaled
+                PNFI <- (df/df.null) * t1/t2
+                indices["pnfi.scaled"] <- PNFI
+            }
+
+            # IFI - incremental fit index (Bollen, 1989; Joreskog & Sorbom, 1993)
+            if("ifi" %in% fit.measures) {
+                t1 <- X2.null - X2
+                t2 <- X2.null - df
+                if(t2 < 0) {
+                    IFI <- 1
+                } else {
+                    IFI <- t1/t2
+                }
+                indices["ifi"] <- IFI
+            }
+            if("ifi.scaled" %in% fit.measures) {
+                t1 <- X2.null.scaled - X2.scaled
+                t2 <- X2.null.scaled
+                if(t2 < 0) {
+                    IFI <- 1
+                } else {
+                    IFI <- t1/t2
+                }
+                indices["ifi.scaled"] <- IFI
+            }
+ 
+            # RNI - relative noncentrality index (McDonald & Marsh, 1990)
+            if("rni" %in% fit.measures) {
+                t1 <- X2 - df
+                t2 <- X2.null - df.null
+                if(t1 < 0 || t2 < 0) {
+                    RNI <- 1
+                } else {
+                    RNI <- 1 - t1/t2
+                }
+                indices["rni"] <- RNI
+            }
+            if("rni.scaled" %in% fit.measures) {
+                t1 <- X2.scaled - df.scaled
+                t2 <- X2.null.scaled - df.null.scaled
+                t2 <- X2.null - df.null
+                if(t1 < 0 || t2 < 0) {
+                    RNI <- 1
+                } else {
+                    RNI <- 1 - t1/t2
+                }
+                indices["rni.scaled"] <- RNI
             }
-            indices["tli.scaled"] <- TLI
         }
     }
 
@@ -330,7 +503,9 @@ fitMeasures <- fitmeasures <- function(object, fit.measures="all") {
     N.RMSEA <- max(N, X2*4) # FIXME: good strategy??
     if(any(c("rmsea","rmsea.scaled") %in% fit.measures)) {
         # RMSEA
-        if(df > 0) {
+        if(is.na(X2) || is.na(df)) {
+            RMSEA <- RMSEA.scaled <- as.numeric(NA)
+        } else if(df > 0) {
             if(scaled) {
                 d <- sum(object@Fit@test[[2]]$trace.UGamma)
             } 
@@ -363,7 +538,9 @@ fitMeasures <- fitmeasures <- function(object, fit.measures="all") {
         lower.lambda <- function(lambda) {
             (pchisq(X2, df=df, ncp=lambda) - 0.95)
         }
-        if(df < 1 || lower.lambda(0) < 0.0) {
+        if(is.na(X2) || is.na(df)) {
+            indices["rmsea.ci.lower"] <- NA
+        } else if(df < 1 || lower.lambda(0) < 0.0) {
             indices["rmsea.ci.lower"] <- 0
         } else {
             lambda.l <- try(uniroot(f=lower.lambda, lower=0, upper=X2)$root)
@@ -392,7 +569,9 @@ fitMeasures <- fitmeasures <- function(object, fit.measures="all") {
         lower.lambda <- function(lambda) {
             (pchisq(XX2, df=df2, ncp=lambda) - 0.95)
         }
-        if(df < 1 || df2 < 1 || lower.lambda(0) < 0.0) {
+        if(is.na(XX2) || is.na(df2)) {
+            indices["rmsea.ci.lower.scaled"] <- NA
+        } else if(df < 1 || df2 < 1 || lower.lambda(0) < 0.0) {
             indices["rmsea.ci.lower.scaled"] <- 0
         } else {
             lambda.l <- try(uniroot(f=lower.lambda, lower=0, upper=XX2)$root)
@@ -411,7 +590,9 @@ fitMeasures <- fitmeasures <- function(object, fit.measures="all") {
         upper.lambda <- function(lambda) {
             (pchisq(X2, df=df, ncp=lambda) - 0.05)
         }
-        if(df < 1 || upper.lambda(N.RMSEA) > 0 || upper.lambda(0) < 0) {
+        if(is.na(X2) || is.na(df)) {
+            indices["rmsea.ci.upper"] <- NA
+        } else if(df < 1 || upper.lambda(N.RMSEA) > 0 || upper.lambda(0) < 0) {
             indices["rmsea.ci.upper"] <- 0
         } else {
             lambda.u <- try(uniroot(f=upper.lambda, lower=0, upper=N.RMSEA)$root)
@@ -437,8 +618,10 @@ fitMeasures <- fitmeasures <- function(object, fit.measures="all") {
         upper.lambda <- function(lambda) {
             (pchisq(XX2, df=df2, ncp=lambda) - 0.05)
         }
-        if(df < 1 || df2 < 1 || upper.lambda(N.RMSEA) > 0 || 
-                                upper.lambda(0) < 0) {
+        if(is.na(XX2) || is.na(df2)) {
+            indices["rmsea.ci.upper.scaled"] <- NA
+        } else if(df < 1 || df2 < 1 || upper.lambda(N.RMSEA) > 0 || 
+                                       upper.lambda(0) < 0) {
             indices["rmsea.ci.upper.scaled"] <- 0
         } else {
             lambda.u <- try(uniroot(f=upper.lambda, lower=0, upper=N.RMSEA)$root)
@@ -455,7 +638,9 @@ fitMeasures <- fitmeasures <- function(object, fit.measures="all") {
     }    
 
     if("rmsea.pvalue" %in% fit.measures) {
-        if(df > 0) {
+        if(is.na(X2) || is.na(df)) {
+            indices["rmsea.pvalue"] <- as.numeric(NA)
+        } else if(df > 0) {
             if(object@Options$mimic %in% c("lavaan","Mplus")) {
                 GG <- 0
                 ncp <- (N-GG)*df*0.05^2/G
@@ -478,7 +663,9 @@ fitMeasures <- fitmeasures <- function(object, fit.measures="all") {
             XX2 <- X2
             df2 <- sum(object@Fit@test[[2]]$trace.UGamma)
         }
-        if(df > 0) {
+        if(is.na(XX2) || is.na(df2)) {
+            indices["rmsea.pvalue.scaled"] <- as.numeric(NA)
+        } else if(df > 0) {
             if(object@Options$mimic %in% c("lavaan", "Mplus")) {
                 GG <- 0
                 ncp <- (N-GG)*df2*0.05^2/G
@@ -493,8 +680,10 @@ fitMeasures <- fitmeasures <- function(object, fit.measures="all") {
         }
     }
 
-    if("srmr" %in% fit.measures) {
-        # SRMR
+    if(any(c("rmr","srmr") %in% fit.measures)) {
+        # RMR and SRMR
+        rmr.group <- numeric(G)
+        rmr_nomean.group <- numeric(G)
         srmr.group <- numeric(G)
         srmr_nomean.group <- numeric(G)
         for(g in 1:G) {
@@ -518,6 +707,7 @@ fitMeasures <- fitmeasures <- function(object, fit.measures="all") {
             sqrt.d <- 1/sqrt(diag(S))
             D <- diag(sqrt.d, ncol=length(sqrt.d))
             R <- D %*% (S - Sigma.hat) %*% D
+            RR <- (S - Sigma.hat) # not standardized
 
             # this is what the Mplus documentation suggest, 
             # but is not what is used!
@@ -528,16 +718,20 @@ fitMeasures <- fitmeasures <- function(object, fit.measures="all") {
             if(meanstructure) {
                 # standardized residual mean vector
                 R.mean <- D %*% (M - Mu.hat)
+                RR.mean <- (M - Mu.hat) # not standardized
                 #R.mean <-  D %*% M - D2 %*% Mu.hat
                 e <- nvar*(nvar+1)/2 + nvar
                 srmr.group[g] <- sqrt( (sum(R[lower.tri(R, diag=TRUE)]^2) +
                                         sum(R.mean^2))/ e )
+                rmr.group[g] <- sqrt( (sum(RR[lower.tri(RR, diag=TRUE)]^2) +
+                                       sum(RR.mean^2))/ e )
                 e <- nvar*(nvar+1)/2
                 srmr_nomean.group[g] <- sqrt( sum(R[lower.tri(R, diag=TRUE)]^2) / e )
-                srmr_nomean.group[g] 
+                rmr_nomean.group[g] <- sqrt( sum(RR[lower.tri(RR, diag=TRUE)]^2) / e )
             } else {
                 e <- nvar*(nvar+1)/2
                 srmr_nomean.group[g] <- srmr.group[g] <- sqrt( sum(R[lower.tri(R, diag=TRUE)]^2) / e )
+                rmr_nomean.group[g] <- rmr.group[g] <- sqrt( sum(RR[lower.tri(RR, diag=TRUE)]^2) / e )
                 
             }
         }
@@ -546,13 +740,82 @@ fitMeasures <- fitmeasures <- function(object, fit.measures="all") {
             ## FIXME: get the scaling right
             SRMR <- as.numeric( (unlist(object@SampleStats@nobs) %*% srmr.group) / object@SampleStats@ntotal )
             SRMR_NOMEAN <- as.numeric( (unlist(object@SampleStats@nobs) %*% srmr_nomean.group) / object@SampleStats@ntotal )
+            RMR <- as.numeric( (unlist(object@SampleStats@nobs) %*% rmr.group) / object@SampleStats@ntotal )
+            RMR_NOMEAN <- as.numeric( (unlist(object@SampleStats@nobs) %*% rmr_nomean.group) / object@SampleStats@ntotal )
         } else {
             SRMR <- srmr.group[1]
             SRMR_NOMEAN <- srmr_nomean.group[1]
+            RMR <- rmr.group[1]
+            RMR_NOMEAN <- rmr_nomean.group[1]
         }
 
         indices["srmr"] <- SRMR
         indices["srmr_nomean"] <- SRMR_NOMEAN
+        indices["rmr"] <- RMR
+        indices["rmr_nomean"] <- RMR_NOMEAN
+    }
+
+    if(any(c("cn_05", "cn_01") %in% fit.measures)) {
+        CN_05 <- qchisq(p=0.95, df=df)/(X2/N) + 1
+        CN_01 <- qchisq(p=0.99, df=df)/(X2/N) + 1
+        indices["cn_05"] <- CN_05
+        indices["cn_01"] <- CN_01
+    }
+
+    if(any(c("gfi","agfi","pgfi") %in% fit.measures)) {
+        gfi.group <- numeric(G)
+        WLS.obs <- object@SampleStats@WLS.obs
+        WLS.V   <- getWLS.V(object)
+        WLS.est <- getWLS.est(object)
+        for(g in 1:G) {
+            wls.obs <- WLS.obs[[g]]
+            wls.est <- WLS.est[[g]]
+            wls.v   <- WLS.V[[g]]
+            
+            if(is.null(wls.v)) {
+                gfi.group[g] <- as.numeric(NA)
+            } else {
+                wls.diff <- wls.obs - wls.est
+                t1 <- crossprod(wls.diff, wls.v) %*% wls.diff
+                t2 <- crossprod(wls.obs, wls.v) %*% wls.obs
+                gfi.group[g] <- 1 - t1/t2
+            }
+        }
+        if(G > 1) {
+            ## FIXME: get the scaling right
+            GFI <- as.numeric( (unlist(object@SampleStats@nobs) %*% gfi.group) / object@SampleStats@ntotal )
+        } else {
+            GFI <- gfi.group[1L]
+        }
+        indices["gfi"] <- GFI
+        nel <- length(unlist(WLS.obs)) # total number of modeled sample stats
+        if(is.na(df)) {
+            indices["agfi"] <- as.numeric(NA)
+        } else if(df > 0) {
+            indices["agfi"] <- 1 - (nel/df) * (1 - GFI)
+        } else {
+            indices["agfi"] <- 1
+        }
+        # LISREL formula (Simplis book 2002, p. 126)
+        indices["pgfi"] <- (df/nel)*GFI 
+    }
+
+    # MFI - McDonald Fit Index (McDonald, 1989)
+    if("mfi" %in% fit.measures) { 
+        #MFI <- exp(-0.5 * (X2 - df)/(N-1)) # Hu & Bentler 1998 Table 1
+        MFI <- exp(-0.5 * (X2 - df)/N)
+        indices["mfi"] <- MFI
+    }
+
+    # ECVI - cross-validation index (Brown & Cudeck, 1989)
+    # not defined for multiple groups and/or models with meanstructures
+    if("ecvi" %in% fit.measures) {
+        if(G > 1 || meanstructure) {
+            ECVI <- as.numeric(NA)
+        } else {
+            ECVI <- X2/N + (2*npar)/N
+        }
+        indices["ecvi"] <- ECVI
     }
 
     if("ntotal" %in% fit.measures) {
@@ -581,13 +844,15 @@ fitMeasures <- fitmeasures <- function(object, fit.measures="all") {
 # print a nice summary of the fit measures
 print.fit.measures <- function(x) {
 
+   names.x <- names(x)
+
    # scaled?
-   scaled <- "chisq.scaled" %in% names(x)
+   scaled <- "chisq.scaled" %in% names.x
 
    # independence model
-   if("baseline.chisq" %in% names(x)) {
-       cat("Chi-square test baseline model:\n\n")
-       t0.txt <- sprintf("  %-40s", "Minimum Function Chi-square")
+   if("baseline.chisq" %in% names.x) {
+       cat("Model test baseline model:\n\n")
+       t0.txt <- sprintf("  %-40s", "Minimum Function Test Statistic")
        t1.txt <- sprintf("  %10.3f", x["baseline.chisq"])
        t2.txt <- ifelse(scaled,
                  sprintf("  %10.3f", x["baseline.chisq.scaled"]), "")
@@ -610,24 +875,69 @@ print.fit.measures <- function(x) {
        cat(t0.txt, t1.txt, t2.txt, "\n", sep="")
     }
 
-   # cfi/tli
-   if("cfi" %in% names(x)) {
-       cat("\nFull model versus baseline model:\n\n")
-       t0.txt <- sprintf("  %-40s", "Comparative Fit Index (CFI)")
-       t1.txt <- sprintf("  %10.3f", x["cfi"])
-       t2.txt <- ifelse(scaled,
-                 sprintf("  %10.3f", x["cfi.scaled"]), "")
-       cat(t0.txt, t1.txt, t2.txt, "\n", sep="")
+    # cfi/tli
+    if(any(c("cfi","tli","nnfi","rfi","nfi","ifi","rni","pnfi") %in% names.x)) {
+        cat("\nFull model versus baseline model:\n\n")
 
-       t0.txt <- sprintf("  %-40s", "Tucker-Lewis Index (TLI)")
-       t1.txt <- sprintf("  %10.3f", x["tli"])
-       t2.txt <- ifelse(scaled,
-                 sprintf("  %10.3f", x["tli.scaled"]), "")
-       cat(t0.txt, t1.txt, t2.txt, "\n", sep="")
-   }
+        if("cfi" %in% names.x) {
+            t0.txt <- sprintf("  %-40s", "Comparative Fit Index (CFI)")
+            t1.txt <- sprintf("  %10.3f", x["cfi"])
+            t2.txt <- ifelse(scaled, sprintf("  %10.3f", x["cfi.scaled"]), "")
+            cat(t0.txt, t1.txt, t2.txt, "\n", sep="")
+        }
+
+        if("tli" %in% names.x) {
+            t0.txt <- sprintf("  %-40s", "Tucker-Lewis Index (TLI)")
+            t1.txt <- sprintf("  %10.3f", x["tli"])
+            t2.txt <- ifelse(scaled, sprintf("  %10.3f", x["tli.scaled"]), "")
+            cat(t0.txt, t1.txt, t2.txt, "\n", sep="")
+        }
+
+        if("nnfi" %in% names.x) {
+            t0.txt <- sprintf("  %-42s", "Bentler-Bonett Non-normed Fit Index (NNFI)")
+            t1.txt <- sprintf("  %8.3f", x["nnfi"])
+            t2.txt <- ifelse(scaled, sprintf("  %10.3f", x["nnfi.scaled"]), "")
+            cat(t0.txt, t1.txt, t2.txt, "\n", sep="")
+        }
+ 
+        if("nfi" %in% names.x) {
+            t0.txt <- sprintf("  %-40s", "Bentler-Bonett Normed Fit Index (NFI)")
+            t1.txt <- sprintf("  %10.3f", x["nfi"])
+            t2.txt <- ifelse(scaled, sprintf("  %10.3f", x["nfi.scaled"]), "")
+            cat(t0.txt, t1.txt, t2.txt, "\n", sep="")
+        }
+  
+        if("nfi" %in% names.x) {
+            t0.txt <- sprintf("  %-40s", "Parsimony Normed Fit Index (PNFI)")
+            t1.txt <- sprintf("  %10.3f", x["pnfi"])
+            t2.txt <- ifelse(scaled, sprintf("  %10.3f", x["pnfi.scaled"]), "")
+            cat(t0.txt, t1.txt, t2.txt, "\n", sep="")
+        }
+
+        if("rfi" %in% names.x) {
+            t0.txt <- sprintf("  %-40s", "Bollen's Relative Fit Index (RFI)")
+            t1.txt <- sprintf("  %10.3f", x["rfi"])
+            t2.txt <- ifelse(scaled, sprintf("  %10.3f", x["rfi.scaled"]), "")
+            cat(t0.txt, t1.txt, t2.txt, "\n", sep="")
+        }
+
+        if("ifi" %in% names.x) {
+            t0.txt <- sprintf("  %-40s", "Bollen's Incremental Fit Index (IFI)")
+            t1.txt <- sprintf("  %10.3f", x["ifi"])
+            t2.txt <- ifelse(scaled, sprintf("  %10.3f", x["ifi.scaled"]), "")
+            cat(t0.txt, t1.txt, t2.txt, "\n", sep="")
+        }
+
+        if("rni" %in% names.x) {
+            t0.txt <- sprintf("  %-40s", "Relative Noncentrality Index (RNI)")
+            t1.txt <- sprintf("  %10.3f", x["rni"])
+            t2.txt <- ifelse(scaled, sprintf("  %10.3f", x["rni.scaled"]), "")
+            cat(t0.txt, t1.txt, t2.txt, "\n", sep="")
+        }
+    }
 
    # likelihood
-   if("logl" %in% names(x)) {
+   if("logl" %in% names.x) {
        cat("\nLoglikelihood and Information Criteria:\n\n")
        t0.txt <- sprintf("  %-40s", "Loglikelihood user model (H0)")
        t1.txt <- sprintf("  %10.3f", x["logl"])
@@ -686,14 +996,14 @@ print.fit.measures <- function(x) {
    }
 
    # RMSEA
-   if("rmsea" %in% names(x)) {
+   if("rmsea" %in% names.x) {
        cat("\nRoot Mean Square Error of Approximation:\n\n")
        t0.txt <- sprintf("  %-40s", "RMSEA")
        t1.txt <- sprintf("  %10.3f", x["rmsea"])
        t2.txt <- ifelse(scaled,
                  sprintf("  %10.3f", x["rmsea.scaled"]), "")
        cat(t0.txt, t1.txt, t2.txt, "\n", sep="")
-       if("rmsea.ci.lower" %in% names(x)) {
+       if("rmsea.ci.lower" %in% names.x) {
            t0.txt <- sprintf("  %-38s", "90 Percent Confidence Interval")
            t1.txt <- sprintf("  %5.3f", x["rmsea.ci.lower"])
            t2.txt <- sprintf("  %5.3f", x["rmsea.ci.upper"])
@@ -702,7 +1012,7 @@ print.fit.measures <- function(x) {
                                                x["rmsea.ci.upper.scaled"]), "")
            cat(t0.txt, t1.txt, t2.txt, t3.txt, "\n", sep="")
        }
-       if("rmsea.pvalue" %in% names(x)) {
+       if("rmsea.pvalue" %in% names.x) {
            t0.txt <- sprintf("  %-40s", "P-value RMSEA <= 0.05")
            t1.txt <- sprintf("  %10.3f", x["rmsea.pvalue"])
            t2.txt <- ifelse(scaled,
@@ -711,18 +1021,95 @@ print.fit.measures <- function(x) {
        }
    }
 
-   # SRMR
-   if("srmr" %in% names(x)) {
-       cat("\nStandardized Root Mean Square Residual:\n\n")
-       t0.txt <- sprintf("  %-40s", "SRMR")
-       t1.txt <- sprintf("  %10.3f", x["srmr"])
-       t2.txt <- ifelse(scaled,
-                 sprintf("  %10.3f", x["srmr"]), "")
-       cat(t0.txt, t1.txt, t2.txt, "\n", sep="")
-       #cat(t0.txt, t1.txt, "\n", sep="")
-   }
+    # SRMR
+    if(any(c("rmr","srmr") %in% names.x)) {
+        cat("\nStandardized Root Mean Square Residual:\n\n")
 
-   cat("\n")
+        if("rmr" %in% names.x) {
+            t0.txt <- sprintf("  %-40s", "RMR")
+            t1.txt <- sprintf("  %10.3f", x["rmr"])
+            t2.txt <- ifelse(scaled, sprintf("  %10.3f", x["rmr"]), "")
+            cat(t0.txt, t1.txt, t2.txt, "\n", sep="")
+        }
+        if("rmr_nomean" %in% names.x) {
+            t0.txt <- sprintf("  %-40s", "RMR (No Mean)")
+            t1.txt <- sprintf("  %10.3f", x["rmr_nomean"])
+            t2.txt <- ifelse(scaled, sprintf("  %10.3f", x["rmr_nomean"]), "")
+            cat(t0.txt, t1.txt, t2.txt, "\n", sep="")
+        }
+        if("srmr" %in% names.x) {
+            t0.txt <- sprintf("  %-40s", "SRMR")
+            t1.txt <- sprintf("  %10.3f", x["srmr"])
+            t2.txt <- ifelse(scaled, sprintf("  %10.3f", x["srmr"]), "")
+            cat(t0.txt, t1.txt, t2.txt, "\n", sep="")
+        }
+        if("srmr_nomean" %in% names.x) {
+            t0.txt <- sprintf("  %-40s", "SRMR (No Mean)")
+            t1.txt <- sprintf("  %10.3f", x["srmr_nomean"])
+            t2.txt <- ifelse(scaled, sprintf("  %10.3f", x["srmr_nomean"]), "")
+            cat(t0.txt, t1.txt, t2.txt, "\n", sep="")
+        }
+    }
+
+    # Other
+    if(any(c("cn_05","cn_01","gfi","agfi","pgfi","mfi") %in% names.x)) {
+        cat("\nOther Fit Indices:\n\n")
+
+        if("cn_05" %in% names.x) {
+            t0.txt <- sprintf("  %-40s", "Hoelter Critical N (CN) alpha=0.05")
+            t1.txt <- sprintf("  %10.3f", x["cn_05"])
+            t2.txt <- ifelse(scaled, sprintf("  %10.3f", x["cn_05"]), "")
+            cat(t0.txt, t1.txt, t2.txt, "\n", sep="")
+        }
+        if("cn_01" %in% names.x) {
+            t0.txt <- sprintf("  %-40s", "Hoelter Critical N (CN) alpha=0.01")
+            t1.txt <- sprintf("  %10.3f", x["cn_01"])
+            t2.txt <- ifelse(scaled, sprintf("  %10.3f", x["cn_01"]), "")
+            cat(t0.txt, t1.txt, t2.txt, "\n", sep="")
+        }
+        if(any(c("cn_05", "cn_01") %in% names.x)) {
+            cat("\n")
+        }
+        if("gfi" %in% names.x) {
+            t0.txt <- sprintf("  %-40s", "Goodness of Fit Index (GFI)")
+            t1.txt <- sprintf("  %10.3f", x["gfi"])
+            t2.txt <- ifelse(scaled, sprintf("  %10.3f", x["gfi"]), "")
+            cat(t0.txt, t1.txt, t2.txt, "\n", sep="")
+        }
+        if("agfi" %in% names.x) {
+            t0.txt <- sprintf("  %-40s", "Adjusted Goodness of Fit Index (AGFI)")
+            t1.txt <- sprintf("  %10.3f", x["agfi"])
+            t2.txt <- ifelse(scaled, sprintf("  %10.3f", x["agfi"]), "")
+            cat(t0.txt, t1.txt, t2.txt, "\n", sep="")
+        }
+        if("pgfi" %in% names.x) {
+            t0.txt <- sprintf("  %-40s", "Parsimony Goodness of Fit Index (PGFI)")
+            t1.txt <- sprintf("  %10.3f", x["pgfi"])
+            t2.txt <- ifelse(scaled, sprintf("  %10.3f", x["pgfi"]), "")
+            cat(t0.txt, t1.txt, t2.txt, "\n", sep="")
+        }
+        if(any(c("gfi","agfi","pgfi") %in% names.x)) {
+            cat("\n")
+        }
+        if("mfi" %in% names.x) {
+            t0.txt <- sprintf("  %-40s", "McDonald Fit Index (MFI)")
+            t1.txt <- sprintf("  %10.3f", x["mfi"])
+            t2.txt <- ifelse(scaled, sprintf("  %10.3f", x["mfi"]), "")
+            cat(t0.txt, t1.txt, t2.txt, "\n", sep="")
+        }
+        if("mfi" %in% names.x) {
+            cat("\n")
+        }
+        if("ecvi" %in% names.x) {
+            t0.txt <- sprintf("  %-40s", "Expected Cross-Validation Index (ECVI)")
+            t1.txt <- sprintf("  %10.3f", x["ecvi"])
+            t2.txt <- ifelse(scaled, sprintf("  %10.3f", x["ecvi"]), "")
+            cat(t0.txt, t1.txt, t2.txt, "\n", sep="")
+        }
+
+    }
+
+    cat("\n")
 }
 
 
