@@ -12,7 +12,8 @@ lavaanNames <- function(object, type="ov", group=NULL) {
 }
 
 # internal version
-vnames <- function(partable, type=NULL, group=NULL, warn=FALSE) {
+vnames <- function(partable, type=NULL, group=NULL, warn=FALSE,
+                   ov.x.fatal=FALSE) {
 
     stopifnot(is.list(partable), !missing(type),
               type %in% c("lv",   "ov", "lv.regular",
@@ -106,6 +107,11 @@ vnames <- function(partable, type=NULL, group=NULL, warn=FALSE) {
         }
         idx.no.x <- which(ov.x %in% vars)
         if(length(idx.no.x)) {
+            if(ov.x.fatal) {
+                stop("lavaan ERROR: model syntax contains variance/covariance/intercept formulas\n  involving (an) exogenous variable(s): [", 
+                        paste(ov.x[idx.no.x], collapse=" "),
+                        "];\n  Please remove them and try again.")
+            }
             if(warn) {
                 warning("lavaan WARNING: model syntax contains variance/covariance/intercept formulas\n  involving (an) exogenous variable(s): [", 
                         paste(ov.x[idx.no.x], collapse=" "),
@@ -584,6 +590,7 @@ getLIST <- function(FLAT=NULL,
 
     # LATENT RESPONSE SCALES (DELTA)
     if(auto.delta && auto.th && length(ov.names.ord) > 0L && 
+       length(lv.names) > 0L &&
        (ngroups > 1L || any(FLAT$op == "~*~"))) {
         lhs <- c(lhs, ov.names.ord)
         rhs <- c(rhs, ov.names.ord)
@@ -1033,7 +1040,11 @@ independenceModel <- function(ov.names=NULL, ov=NULL,
 # and af(x) returns the evaluated restrictions under M0).
 # The 'jacobian' of this function 'A' will be used in the anova
 # anova() function, and elsewhere
-getConstraintsFunction <- function(p1, p0, ceq.function=NULL) {
+getConstraintsFunction <- function(p1, p0) {
+
+    # check for inequality constraints
+    if(any(c(p0$op,p1$op) %in% c(">","<"))) 
+        stop("lavaan ERROR: anova() can not handle inequality constraints; use InformativeTesting() instead")
 
     npar.p1 <- max(p1$free)
     npar.p0 <- max(p0$free)
@@ -1085,6 +1096,83 @@ getConstraintsFunction <- function(p1, p0, ceq.function=NULL) {
         }
     }
 
+    # add all NEW constraints using "==" (not already in p1)
+    p0.con.idx <- which(p0$op == "==")
+    if(length(p0.con.idx) > 0L) {
+        # first, remove those also present in p1
+        del.idx <- integer(0L)
+        for(con in 1:length(p0.con.idx)) {
+            p0.idx <- p0.con.idx[con]
+            p1.idx <- which(p1$op == "==" &
+                            p1$lhs == p0$lhs[p0.idx] &
+                            p1$rhs == p0$rhs[p0.idx])
+            if(length(p1.idx) > 0L)
+                del.idx <- c(del.idx, con)
+        }
+        if(length(del.idx) > 0L)
+            p0.con.idx <- p0.con.idx[-del.idx]   
+    }
+    eq.idx <- p0.con.idx
+    if(length(eq.idx) > 0L) {
+        def.idx <- which(p0$op == ":=")
+        # first come the variable definitions
+        if(length(def.idx) > 0L) {
+            for(i in 1:length(def.idx)) {
+                lhs <- p0$lhs[ def.idx[i] ]
+                rhs <- p0$rhs[ def.idx[i] ]
+                def.string <- rhs
+                # coerce to expression to extract variable names
+                def.labels <- all.vars( parse(file="", text=def.string) )
+                # get corresponding 'x' indices
+                def.x.idx  <- p0$free[match(def.labels, p0$label)]
+                def.x.lab  <- paste("x[", def.x.idx, "]",sep="")
+                # put both the labels and the expression in the function BODY
+                BODY.txt <- paste(BODY.txt,
+                    paste(def.labels, "=",def.x.lab, collapse=";"),"\n",
+                    lhs, " = ", def.string, "\n", sep="")
+            }
+        }
+
+        for(i in 1:length(eq.idx)) {
+            lhs <- p0$lhs[ eq.idx[i] ]
+            rhs <- p0$rhs[ eq.idx[i] ]
+            if(rhs == "0") {
+                eq.string <- lhs
+            } else {
+                eq.string <- paste(lhs, "- (", rhs, ")", sep="")
+            }
+            # coerce to expression to extract variable names
+            eq.labels <- all.vars( parse(file="", text=eq.string) )
+            # get corresponding 'x' indices
+            if(length(def.idx) > 0L) {
+                # remove def.names from ineq.labels
+                def.names <- as.character(p0$lhs[def.idx])
+                d.idx <- which(eq.labels %in% def.names)
+                if(length(d.idx) > 0) eq.labels <- eq.labels[-d.idx]
+            }
+            if(length(eq.labels) > 0L) {
+                eq.x.idx  <- p0$free[match(eq.labels, p0$label)]
+                if(any(is.na(eq.x.idx))) {
+                    stop("lavaan ERROR: unknown label(s) in equality constraint: ",
+                         paste(eq.labels[which(is.na(eq.x.idx))], collapse=" "))
+                }
+                if(any(eq.x.idx == 0)) {
+                    stop("lavaan ERROR: non-free parameter(s) in inequality constraint: ",
+                        paste(eq.labels[which(eq.x.idx == 0)], collapse=" "))
+                }
+                eq.x.lab  <- paste("x[", eq.x.idx, "]",sep="")
+                # put both the labels and the expression in the function BODY
+                BODY.txt <- paste(BODY.txt,
+                    paste(eq.labels, "=", eq.x.lab, collapse=";"),"\n",
+                    "out[", i, "] = ", eq.string, "\n", sep="")
+            } else {
+                BODY.txt <- paste(BODY.txt,
+                    "out[", i, "] = ", eq.string, "\n", sep="")
+            }
+        }
+               
+    }
+
     # wrap function
     BODY.txt <- paste(BODY.txt, "return(out)\n}\n", sep="")
     body(con.function) <- parse(file="", text=BODY.txt)
